import abc
from typing import Union, List, Optional

from aup_manager.models import Admin, Entity


class ConnectorInterface(metaclass=abc.ABCMeta):
    @classmethod
    def __subclasshook__(cls, subclass):
        return (
            hasattr(subclass, "get_admin")
            and callable(subclass.get_admin)
            and hasattr(subclass, "get_relevant_entity_id_types")
            and callable(subclass.get_relevant_entity_id_types)
            and hasattr(subclass, "get_entities_for_admin")
            and callable(subclass.get_entities_for_admin)
            and hasattr(subclass, "get_user_id")
            and callable(subclass.get_user_id)
            or NotImplementedError
        )

    @abc.abstractmethod
    def get_admin(self, ext_login: str, ext_name: str = None) -> Optional[Admin]:
        """Get User object from external source"""
        raise NotImplementedError

    @abc.abstractmethod
    def get_relevant_entity_id_types(
        self, entity_type_id: str, user_id: Union[str, int]
    ) -> List[str]:
        """Get entity type_ids which are connected to given entity"""
        raise NotImplementedError

    @abc.abstractmethod
    def get_entities_for_admin(
        self, admin_uid: Union[str, int] = None
    ) -> dict[str, List[Entity]]:
        """Get dict[type, List[Entity]] to which can admin assign Aups"""
        raise NotImplementedError

    @abc.abstractmethod
    def get_user_id(self, ext_login: str, ext_name: str = None) -> Union[int, str]:
        """Get user id from external source"""
        raise NotImplementedError



from aup_manager.flask_app import get_user_accepted_aups
from aup_manager.flask_app import get_all_user_accepted_aups
from aup_manager.flask_app import get_user_not_accepted_aups
from aup_manager.flask_app import user_accepted_aups
from aup_manager.flask_app import token_info

__all__ = [
    "get_user_accepted_aups",
    "get_all_user_accepted_aups",
    "get_user_not_accepted_aups",
    "user_accepted_aups",
    "token_info",
]



from typing import Union, List, Optional

from perun.connector import AdaptersManager
from perun.connector.adapters.AdaptersManager import AdaptersManagerNotExistsException

from aup_manager.connectors.ConnectorInterface import ConnectorInterface
from aup_manager.models import Entity, Admin


class PerunConnector(ConnectorInterface):
    def __init__(self, config):
        self.connector = AdaptersManager(config, attrs_map={})

    def get_admin(self, ext_login: str, ext_name: str = None) -> Optional[Admin]:
        """
        :param ext_login: user login
        :param ext_name: external source name
        :return: Admin object with perun user id and login
        """
        try:
            user = self.connector.get_perun_user(ext_name, [ext_login])
        except AdaptersManagerNotExistsException:
            return None
        return Admin(str(user.id), ext_login)

    def get_relevant_entity_id_types(
        self, entity_type_id: str, user_id: Union[str, int]
    ) -> List[str]:
        """
        :param entity_type_id: entity type with its id (facility:1)
        :param user_id: id of user
        :return: List of type:id of all entities related to given facility.
        Related entities are all resources and their VOs on facility, together with
        groups assigned to them which the user is a member of.
        (list also includes entity_type_id passed in parameter)
        """
        entity_type, entity_id = entity_type_id.split(":", 2)
        if entity_type != "facility":
            raise ValueError("Entity type has to be facility")
        result = {entity_type_id}

        resources = self.connector.get_resources_for_facility(int(entity_id), False)
        for resource in resources:
            if not self.connector.get_groups_for_resource(resource, False):
                result |= {f"resource:{resource.id}", f"vo:{resource.vo.id}"}
            else:
                groups = self.connector.get_groups_where_user_is_active_resource(
                    int(user_id), resource, False
                )
                if not groups:
                    continue
                result |= {f"resource:{resource.id}", f"vo:{resource.vo.id}"}
                for group in groups:
                    result.add(f"group:{group.id}")
                    for parent_group in self.connector.get_all_parent_groups_for_group(
                        group.id, False
                    ):
                        result.add(f"group:{parent_group.id}")

        return list(result)

    def get_entities_for_admin(
        self, admin_uid: Union[str, int] = None
    ) -> dict[str, List[Entity]]:
        """
        :param admin_uid: id of User
        :return: List of all entities to which given user has admin rights.
        """
        if isinstance(admin_uid, str):
            admin_uid = int(admin_uid)
        return {
            "resource": self._get_resources_from_perun_as_entities(admin_uid),
            "vo": self._get_vo_from_perun_as_entities(admin_uid),
            "facility": self._get_facilities_from_perun_as_entities(admin_uid),
            "group": self._get_groups_from_perun_as_entities(admin_uid),
        }

    def _get_resources_from_perun_as_entities(
        self, admin_uid: Union[str, int] = None
    ) -> List[Entity]:
        resources = self.connector.get_resources_where_user_is_admin(admin_uid)
        return [
            Entity(resource.id, resource.name, "resource") for resource in resources
        ]

    def _get_vo_from_perun_as_entities(
        self, admin_uid: Union[str, int] = None
    ) -> List[Entity]:
        vos = self.connector.get_vos_where_user_is_admin(admin_uid)
        return [Entity(vo.id, vo.name, "vo") for vo in vos]

    def _get_facilities_from_perun_as_entities(
        self, admin_uid: Union[str, int] = None
    ) -> List[Entity]:
        facilities = self.connector.get_facilities_where_user_is_admin(admin_uid)
        return [
            Entity(facility.id, facility.name, "facility") for facility in facilities
        ]

    def _get_groups_from_perun_as_entities(
        self, admin_uid: Union[str, int] = None
    ) -> List[Entity]:
        groups = self.connector.get_groups_where_user_is_admin(admin_uid)
        return [Entity(group.id, group.name, "group") for group in groups]

    def get_user_id(self, ext_login: str, ext_name: str = None) -> Union[int, str]:
        user = self.connector.get_perun_user(ext_name, [ext_login])
        return user.id



from typing import Union, List, Optional

from pymongo import MongoClient

from aup_manager.connectors.ConnectorInterface import ConnectorInterface
from aup_manager.models import Admin, Entity


class MongodbConnector(ConnectorInterface):
    def __init__(self, config):
        self.client = MongoClient(config["uri"])
        self.db = self.client[config["db_name"]]
        self.admins_collection = config["admins_collection"]
        self.entities_collection = config["entities_collection"]

    def get_admin(self, ext_login: str, ext_name: str = None) -> Optional[Admin]:
        admin = self.db[self.admins_collection].find_one({"ext_login": ext_login})
        if not admin:
            return None
        return MongodbConnector.__admin_mongo_to_python(admin)

    def get_relevant_entity_id_types(
        self, entity_type_id: str, user_id: Union[str, int]
    ) -> List[str]:
        return [entity_type_id]

    def get_entities_for_admin(
        self, admin_uid: Union[str, int] = None
    ) -> dict[str, List[Entity]]:
        entities = self.db[self.entities_collection].find()
        result = {}
        for entity_m in entities:
            entity = MongodbConnector.__entity_mongo_to_python(entity_m)
            if not result.get(entity.get_entity_type()):
                result[entity.get_entity_type()] = [entity]
            else:
                result[entity.get_entity_type()].append(entity)
        return result

    def get_user_id(self, ext_login: str, ext_name: str = None) -> Union[int, str]:
        return ext_login

    @staticmethod
    def __admin_mongo_to_python(admin_m):
        return Admin(**admin_m)

    @staticmethod
    def __entity_mongo_to_python(entity_m):
        return Entity(entity_m["_id"], entity_m["name"], entity_m["type"])



import base64
import importlib
import json
import time

import requests
import yaml
from connexion import FlaskApp
from flask import Response, render_template, redirect, session, url_for, request
from flask_pyoidc import OIDCAuthentication
from flask_pyoidc.provider_configuration import ProviderConfiguration, ClientMetadata
from flask_pyoidc.user_session import UserSession
from markdown2 import markdown
from swagger_ui_bundle import swagger_ui_3_path
from werkzeug.exceptions import Unauthorized
from jwcrypto import jwk, jwt, jws
from urllib import parse

from aup_manager.connectors.ConnectorInterface import ConnectorInterface
from aup_manager.db.MongodbDatabase import MongodbDatabase
from aup_manager.models import Acceptance, Status, Aup, Admin, Request

with open("/etc/aup-manager.yaml", "r") as yaml_file:
    cfg = yaml.safe_load(yaml_file)

flask_app = FlaskApp(__name__)

DATABASE = MongodbDatabase(cfg["mongodb"])

CONNECTOR_MODULE, _, CONNECTOR_CLASS_NAME = cfg["connector_class"].rpartition(".")
connector_class = getattr(
    importlib.import_module(CONNECTOR_MODULE), CONNECTOR_CLASS_NAME
)
CONNECTOR: ConnectorInterface = connector_class(cfg["connectors"][CONNECTOR_CLASS_NAME])
EXT_SRC_NAME = cfg["connectors"].get("ext_source_name")

HOSTNAME = cfg["hostname"]
INTROSPECT_URL = cfg["OAuth2"]["introspect_url"]
AUTHORIZATION_URL = cfg["OAuth2"]["authorization_url"]
TOKEN_URL = cfg["OAuth2"]["token_url"]
CLIENT_ID = cfg["OAuth2"]["client_id"]
CLIENT_SECRET = cfg["OAuth2"]["client_secret"]

client_metadata = ClientMetadata(client_id=CLIENT_ID, client_secret=CLIENT_SECRET)
flask_app.app.config.update(
    OIDC_REDIRECT_URI=f"{HOSTNAME}/oidc_callback",
    SECRET_KEY=cfg["secret"],
    post_logout_redirect_uris=[f"{HOSTNAME}/logout", HOSTNAME],
)
provider_config = ProviderConfiguration(
    issuer=cfg["OIDC"]["issuer"], client_metadata=client_metadata
)
auth = OIDCAuthentication({"default": provider_config})

ACCEPT_AUPS_MESSAGE = cfg.get(
    "accept_aups_message",
    "Before proceeding to service, you have to accept following acceptable use "
    "policies. These policies restrict the ways in which the service may be used "
    "and set guidelines as to how it should be used.",
)

TOKEN_ALG = cfg["jwks"]["token_alg"]
KEY_ID = cfg["jwks"]["key_id"]
KEYSTORE = cfg["jwks"]["keystore"]

JWK_SET = jwk.JWKSet()
with open(KEYSTORE, "r") as file:
    JWK_SET.import_keyset(file.read())
JSON_WEB_KEY = JWK_SET.get_key(KEY_ID)


def token_info(token):
    b64_client_id_secret = base64.urlsafe_b64encode(
        f"{CLIENT_ID}:" f"{CLIENT_SECRET}".encode()
    ).decode()
    response = requests.get(
        INTROSPECT_URL,
        params={"token": token},
        headers={"Authorization": f"Basic {b64_client_id_secret}"},
    )
    if response.status_code != 200:
        raise Unauthorized
    return response.json()


def exception_handler(error):
    return {
        "detail": str(error),
        "status": 500,
        "title": "Internal Server Error",
    }, 500


def get_user_accepted_aups(user_id, entity_type_id):
    entity_type_ids = CONNECTOR.get_relevant_entity_id_types(entity_type_id, user_id)
    aups = DATABASE.get_user_accepted_aups_by_condition(user_id, entity_type_ids)
    return Response(json.dumps([aup.to_response_dict() for aup in aups]), 200)


def get_all_user_accepted_aups(user_id):
    aups = DATABASE.get_all_user_accepted_aups(user_id)
    return Response(json.dumps([aup.to_response_dict() for aup in aups]), 200)


def get_user_not_accepted_aups(user_id, entity_type_id):
    entity_type_ids = CONNECTOR.get_relevant_entity_id_types(entity_type_id, user_id)
    aups = DATABASE.get_user_not_accepted_aups_by_condition(user_id, entity_type_ids)
    return Response(json.dumps([aup.to_response_dict() for aup in aups]), 200)


def user_accepted_aups(body):
    __save_acceptances(body["user_id"], list(body["aup_ids"]))
    return Response("Success", 200)


def __save_acceptances(user_id, aup_ids):
    acceptances_list = []
    for aup_id in aup_ids:
        acceptance = Acceptance(
            aup_id,
            str(user_id),
            time.strftime("%Y-%m-%d %H:%M:%S %Z", time.localtime()),
        )
        acceptances_list.append(acceptance)
    if acceptances_list:
        DATABASE.insert_acceptances(acceptances_list)


def verify_jwt(token):
    return jwt.JWT(jwt=token, key=JSON_WEB_KEY).claims


@flask_app.route("/accept_aups/<message>")
def accept_aups(message):
    try:
        message = json.loads(verify_jwt(message))
    except jws.InvalidJWSSignature:
        return Response(json.dumps({"fail": "Invalid signature"}), 400)
    user_id = message.get("user_id")
    entity_type_id = message.get("entity_type_id")
    callback_url = message.get("callback_url")
    nonce = message.get("nonce")
    accept_aups_message = message.get("accept_aups_message", ACCEPT_AUPS_MESSAGE)

    if DATABASE.get_request_by_nonce(nonce):
        return Response(json.dumps({"fail": "Replay attack."}), 403)

    if not user_id or not entity_type_id or not callback_url or not nonce:
        return Response(json.dumps({"fail": "Missing request parameter."}), 400)
    entity_type_ids = CONNECTOR.get_relevant_entity_id_types(entity_type_id, user_id)
    aups = DATABASE.get_user_not_accepted_aups_by_condition(user_id, entity_type_ids)
    if len(aups) == 0:
        DATABASE.save_request(Request(nonce, user_id, status=Status.SUCCESS))
        return redirect(callback_url + "?" + parse.urlencode({"nonce": nonce}))
    aups_as_dict = [aup.__dict__ for aup in aups]

    session["accept_user_id"] = user_id
    session["callback_url"] = callback_url
    session["aup_ids"] = [aup["_id"] for aup in aups_as_dict]
    session["nonce"] = nonce

    DATABASE.save_request(Request(nonce, user_id))

    return render_template(
        "accept_aups.html", aups=aups_as_dict, accept_aups_message=accept_aups_message
    )


@flask_app.route("/save_acceptances")
def save_acceptances():
    user_id = session.pop("accept_user_id", None)
    nonce = session.pop("nonce", None)
    callback_url = session.pop("callback_url", None)
    aup_ids = session.pop("aup_ids", None)
    if not user_id or not nonce or not callback_url or not aup_ids:
        return Response(json.dumps({"fail": "Missing attribute in session."}), 400)

    internal_request = DATABASE.get_request_by_nonce(nonce)
    if (
        not internal_request
        or internal_request.status != internal_request.status.FAILURE
    ):
        return Response(json.dumps({"fail": "Invalid nonce."}), 403)
    __save_acceptances(user_id, aup_ids)
    DATABASE.make_request_success(internal_request.get_id())
    return redirect(callback_url + "?" + parse.urlencode({"nonce": nonce}))


@flask_app.route("/get_accept_result/<message>")
def get_accept_result(message):
    try:
        message = json.loads(verify_jwt(message))
    except jws.InvalidJWSSignature:
        return Response(json.dumps({"fail": "Invalid signature"}), 400)
    nonce = message.get("nonce")
    user_id = message.get("user_id")
    if not nonce or not user_id:
        return Response(json.dumps({"fail": "Missing request parameter."}), 400)
    internal_request = DATABASE.get_request_by_nonce(nonce)
    if (
        not internal_request
        or internal_request.user_id != user_id
        or internal_request.status == Status.FAILURE
    ):
        response = cfg["responses"]["failure"]
    elif internal_request.status == Status.SUCCESS:
        DATABASE.make_request_invalid(internal_request.get_id())
        response = cfg["responses"]["success"]
    elif internal_request.status == Status.INVALID:
        response = cfg["responses"]["invalid-request"]
    else:
        response = "error"
    response_dict = {
        "result": response,
        "nonce": internal_request.nonce,
    }
    return Response(json.dumps(response_dict), 200)


@flask_app.route("/")
def admin_login():
    user_session = UserSession(session, "default")
    if user_session.is_authenticated():
        return redirect(url_for("aup_overview"))
    return render_template("login.html")


@flask_app.route("/logout")
@auth.oidc_logout
def logout():
    session.clear()
    return render_template(
        "text_message.html", text_msg="You were successfully logged out."
    )


def gui_conditions_to_type_id(gui_conditions):
    result = []
    for outer in range(len(gui_conditions)):
        result.append([])
        for condition in gui_conditions[outer]:
            result[outer].append(condition["type_id"])
    return result


@flask_app.route("/save_aup", methods=["POST"])
@auth.oidc_auth("default")
def save_aup():
    result = _get_admin()
    if not isinstance(result, Admin):
        return Response(
            json.dumps(
                {
                    "error": "unauthorized",
                    "error_description": "you do not have rights to update this AUP",
                }
            ),
            403,
        )
    body = request.get_json()
    aup = Aup(
        body["name"],
        body["content"],
        markdown(body["content"]),
        gui_conditions_to_type_id(body["conditions"]),
        body.get("entitlement"),
        additional_data=body.get("additional_data"),
    )
    if DATABASE.insert_aup(aup):
        return Response(json.dumps({"redirect_url": url_for("aup_overview")}), 200)
    return Response(json.dumps({"status": "error occurred"}), 500)


@flask_app.route("/update_aup_name", methods=["POST"])
@auth.oidc_auth("default")
def update_aup_name():
    result = _get_admin()
    if not isinstance(result, Admin):
        return Response(
            json.dumps(
                {
                    "error": "unauthorized",
                    "error_description": "you do not have rights to update this AUP",
                }
            ),
            403,
        )
    body = request.get_json()
    DATABASE.set_aup_name(body["_id"], body["new_name"])
    return Response(json.dumps({"status": "ok"}), 200)


@flask_app.route("/update_aup_content", methods=["POST"])
@auth.oidc_auth("default")
def update_aup_content():
    result = _get_admin()
    if not isinstance(result, Admin):
        return Response(
            json.dumps(
                {
                    "error": "unauthorized",
                    "error_description": "you do not have rights to update this AUP",
                }
            ),
            403,
        )
    body = request.get_json()
    inserted_id = DATABASE.update_aup_text(
        body["_id"], body["content"], markdown(body["content"])
    )
    aup = DATABASE.get_aup_by_id(str(inserted_id))
    if aup:
        return Response(json.dumps(aup.to_dict()), 200)
    return Response(json.dumps({"status": "error occurred"}), 500)


@flask_app.route("/update_aup_conditions", methods=["POST"])
@auth.oidc_auth("default")
def update_aup_conditions():
    result = _get_admin()
    if not isinstance(result, Admin):
        return Response(
            json.dumps(
                {
                    "error": "unauthorized",
                    "error_description": "you do not have rights to update this AUP",
                }
            ),
            403,
        )
    body = request.get_json()
    DATABASE.set_aup_conditions(
        body["_id"], gui_conditions_to_type_id(body["conditions"])
    )
    return Response(json.dumps({"status": "ok"}), 200)


@flask_app.route("/delete_aup", methods=["POST"])
@auth.oidc_auth("default")
def delete_aup():
    result = _get_admin()
    if not isinstance(result, Admin):
        return Response(
            json.dumps(
                {
                    "error": "unauthorized",
                    "error_description": "you do not have rights to update this AUP",
                }
            ),
            403,
        )
    body = request.get_json()
    new_acutal_aup_id = DATABASE.delete_aup_by_id(body["_id"])
    return Response(json.dumps({"new_actual_aup_id": str(new_acutal_aup_id)}), 200)


@flask_app.route("/aup_overview")
@auth.oidc_auth("default")
def aup_overview():
    aups = DATABASE.get_all_aups()
    result = _get_admin()
    if not isinstance(result, Admin):
        return result
    admin = result
    return render_template("aup_overview.html", aups=aups, login=admin.ext_login)


@flask_app.route("/create_aup")
@auth.oidc_auth("default")
def create_aup():
    result = _get_admin()
    if not isinstance(result, Admin):
        return result
    admin = result
    entities = CONNECTOR.get_entities_for_admin(admin.get_id())
    entities_as_dict = {}
    for key, value in entities.items():
        new_value = []
        for entity in value:
            new_value.append(entity.__dict__)
        entities_as_dict[key] = new_value

    return render_template(
        "create_aup.html",
        entities=entities_as_dict,
        entity_types=list(entities.keys()),
    )


def find_entity_in_admin_entities_dict(type_id, entities_dict):
    ent_type, ent_id = type_id.split(":", 1)
    entity_list = entities_dict.get(ent_type)
    if not entity_list:
        return None
    for entity in entity_list:
        if entity.type_id == type_id:
            return entity
    return None


@flask_app.route("/manage_aups")
@auth.oidc_auth("default")
def manage_aups():
    result = _get_admin()
    if not isinstance(result, Admin):
        return result
    admin = result
    admin_entities = CONNECTOR.get_entities_for_admin(admin.get_id())

    entities_as_dict = {}
    for key, value in admin_entities.items():
        new_value = []
        for entity in value:
            entity_as_dict = entity.__dict__
            entity_as_dict["type"] = entity.get_entity_type()
            new_value.append(entity_as_dict)
        entities_as_dict[key] = new_value

    aups_dict = {"enabled": [], "disabled": []}
    all_aups = DATABASE.get_all_aups()
    for aup in all_aups:
        aup_as_dict = aup.to_dict()
        enabled = True
        for outer_ind in range(len(aup.conditions)):
            if not enabled:
                break
            for inner_ind in range(len(aup.conditions[outer_ind])):
                entity = find_entity_in_admin_entities_dict(
                    aup.conditions[outer_ind][inner_ind], admin_entities
                )
                if entity:
                    entity_as_dict = entity.__dict__
                    entity_as_dict["type"] = entity.get_entity_type()
                    aup_as_dict["conditions"][outer_ind][inner_ind] = entity_as_dict
                else:
                    enabled = False
                    aups_dict["disabled"].append(aup.to_dict())
                    break
        if enabled:
            aups_dict["enabled"].append(aup_as_dict)
    return render_template(
        "manage_aups.html",
        aups_dict=aups_dict,
        entities=entities_as_dict,
        entity_types=list(admin_entities.keys()),
    )


def _get_admin():
    sub = UserSession(session).userinfo["sub"]
    admin = session.get("admin")
    if admin:
        admin = Admin(**json.loads(admin))
    if not admin or admin.ext_login != sub:
        admin = CONNECTOR.get_admin(sub, EXT_SRC_NAME)
        if not admin:
            return render_template("unauthorized.html", login=sub)
        session["admin"] = admin.to_json()
    return admin


if __name__ == "__main__":
    auth.init_app(flask_app.app)
    flask_app.add_api(
        "openapi-specification.yaml",
        strict_validation=True,
        validate_responses=True,
        arguments={"authorizationUrl": AUTHORIZATION_URL, "tokenUrl": TOKEN_URL},
        options={
            "swagger_ui": True,
            "swagger_path": swagger_ui_3_path,
            "swagger_ui_config": {
                "oauth2RedirectUrl": f"{cfg['hostname']}/ui/oauth2-redirect.html",
                "persistAuthorization": True,
            },
        },
    )
    flask_app.add_error_handler(Exception, exception_handler)
    flask_app.run(port=8080)



import json
from typing import Union, List, Optional

from bson import ObjectId
from pymongo import MongoClient
from pymongo.errors import CollectionInvalid

from aup_manager.db.DatabaseInterface import DatabaseInterface
from aup_manager.models import Aup, Acceptance, Request, Status


class MongodbDatabase(DatabaseInterface):
    def __init__(self, config):
        self.client = MongoClient(config["uri"])
        self.db = self.client[config["db_name"]]
        try:
            self.db.create_collection("aup")
            self.db.create_collection("acceptance")
            self.db.create_collection("request")
        except CollectionInvalid:
            pass

    def insert_aup(self, aup: Aup) -> Union[str, int]:
        delattr(aup, "_id")
        delattr(aup, "acceptances")
        aup_dict = aup.__dict__
        result = self.db["aup"].insert_one(aup_dict)
        return result.inserted_id

    def insert_acceptance(self, acceptance: Acceptance) -> Union[str, int]:
        delattr(acceptance, "_id")
        result = self.db["acceptance"].insert_one(acceptance.__dict__)
        return result.inserted_id

    def insert_acceptances(
        self, acceptances: List[Acceptance]
    ) -> List[Union[str, int]]:
        insert_list = []
        for acceptance in acceptances:
            delattr(acceptance, "_id")
            acceptance.aup_id = ObjectId(acceptance.aup_id)
            insert_list.append(acceptance.__dict__)
        result = self.db["acceptance"].insert_many(insert_list)
        return result.inserted_ids

    def get_all_aups_with_acceptances(self) -> List[Aup]:
        aups = self.db["aup"].aggregate(
            [
                {
                    "$lookup": {
                        "from": "acceptance",
                        "localField": "_id",
                        "foreignField": "aup_id",
                        "as": "acceptances",
                    }
                }
            ]
        )
        return self.__aup_list_mongo_to_python(aups)

    def get_aup_with_acceptances_by_id(self, _id: Union[str, int]) -> Optional[Aup]:
        aup = self.db["aup"].aggregate(
            [
                {"$match": {"_id": ObjectId(_id)}},
                {
                    "$lookup": {
                        "from": "acceptance",
                        "localField": "_id",
                        "foreignField": "aup_id",
                        "as": "acceptances",
                    }
                },
            ]
        )
        aups = self.__aup_list_mongo_to_python(aup)
        if not aups:
            return None
        return aups[0]

    def get_all_aups(self) -> List[Aup]:
        aups = self.db["aup"].find()
        return self.__aup_list_mongo_to_python(aups)

    def get_aups_by_entitlement(self, entitlement: str) -> List[Aup]:
        aups = self.db["aup"].find({"entitlement": entitlement})
        return self.__aup_list_mongo_to_python(aups)

    def get_aup_by_id(self, _id: Union[str, int]) -> Optional[Aup]:
        aup_m = self.db["aup"].find_one({"_id": ObjectId(_id)})
        if not aup_m:
            return None
        return self.__aup_mongo_to_python(aup_m)

    def get_all_user_accepted_aups(self, user_id: Union[int, str]) -> List[Aup]:
        aups = self.db["aup"].aggregate(
            [
                {
                    "$lookup": {
                        "from": "acceptance",
                        "as": "acceptances",
                        "let": {"aup_id": "$_id"},
                        "pipeline": [
                            {
                                "$match": {
                                    "$expr": {
                                        "$and": [
                                            {"$eq": ["$aup_id", "$$aup_id"]},
                                            {"$eq": ["$user_id", user_id]},
                                        ]
                                    }
                                }
                            }
                        ],
                    }
                },
                {"$match": {"acceptances.user_id": user_id}},
            ]
        )
        return self.__aup_list_mongo_to_python(aups)

    def get_aups_by_condition(self, condition: List[str]) -> List[Aup]:
        aups = self.db["aup"].find(
            {
                "conditions": {
                    "$elemMatch": {"$not": {"$elemMatch": {"$nin": condition}}}
                }
            }
        )
        return self.__aup_list_mongo_to_python(aups)

    def get_aups_by_condition_with_acceptances(self, condition: List[str]) -> List[Aup]:
        aups = self.db["aup"].aggregate(
            [
                {
                    "$match": {
                        "conditions": {
                            "$elemMatch": {"$not": {"$elemMatch": {"$nin": condition}}}
                        }
                    }
                },
                {
                    "$lookup": {
                        "from": "acceptance",
                        "localField": "_id",
                        "foreignField": "aup_id",
                        "as": "acceptances",
                    }
                },
            ]
        )
        return self.__aup_list_mongo_to_python(aups)

    def get_user_accepted_aups_by_condition(
        self, user_id: Union[int, str], condition: List[str]
    ) -> List[Aup]:
        aups = self.db["aup"].aggregate(
            [
                {
                    "$match": {
                        "conditions": {
                            "$elemMatch": {"$not": {"$elemMatch": {"$nin": condition}}}
                        }
                    }
                },
                {
                    "$lookup": {
                        "from": "acceptance",
                        "as": "acceptances",
                        "let": {"aup_id": "$_id"},
                        "pipeline": [
                            {
                                "$match": {
                                    "$expr": {
                                        "$and": [
                                            {"$eq": ["$aup_id", "$$aup_id"]},
                                            {"$eq": ["$user_id", user_id]},
                                        ]
                                    }
                                }
                            }
                        ],
                    }
                },
                {"$match": {"acceptances.user_id": user_id}},
            ]
        )
        return self.__aup_list_mongo_to_python(aups)

    def get_user_not_accepted_aups_by_condition(
        self, user_id: Union[int, str], condition: List[str]
    ) -> List[Aup]:
        aups = self.db["aup"].aggregate(
            [
                {
                    "$match": {
                        "actual_aup_id": None,
                        "conditions": {
                            "$elemMatch": {"$not": {"$elemMatch": {"$nin": condition}}}
                        },
                    }
                },
                {
                    "$lookup": {
                        "from": "acceptance",
                        "as": "acceptances",
                        "let": {"aup_id": "$_id"},
                        "pipeline": [
                            {
                                "$match": {
                                    "$expr": {
                                        "$and": [
                                            {"$eq": ["$aup_id", "$$aup_id"]},
                                            {"$eq": ["$user_id", user_id]},
                                        ]
                                    }
                                }
                            }
                        ],
                    }
                },
                {"$match": {"acceptances.user_id": {"$ne": user_id}}},
            ]
        )
        return self.__aup_list_mongo_to_python(aups)

    def delete_aup_by_id(self, aup_id: Union[int, str]):
        self.db["aup"].delete_one({"_id": ObjectId(aup_id)})
        self.db["acceptance"].delete_many({"aup_id": ObjectId(aup_id)})
        highest_aup = (
            self.db["aup"]
            .find({"actual_aup_id": ObjectId(aup_id)})
            .sort("version", -1)
            .limit(1)
        )
        highest_aup = list(highest_aup)
        if highest_aup:
            self.db["aup"].update_one(
                {"_id": highest_aup[0]["_id"]}, {"$set": {"actual_aup_id": None}}
            )
            self.db["aup"].update_many(
                {"actual_aup_id": ObjectId(aup_id)},
                {"$set": {"actual_aup_id": highest_aup[0]["_id"]}},
            )
            return highest_aup[0]["_id"]
        return None

    def delete_all_aups(self):
        self.db["aup"].delete_many({})

    def set_aup_name(self, aup_id: Union[str, int], new_name: str):
        self.db["aup"].update_one(
            {"_id": ObjectId(aup_id)}, {"$set": {"name": new_name}}
        )
        self.db["aup"].update_many(
            {"actual_aup_id": ObjectId(aup_id)}, {"$set": {"name": new_name}}
        )

    def update_aup_text(
        self, aup_id: Union[str, int], markdown: str, html: str
    ) -> Optional[Union[str, int]]:
        aup = self.get_aup_by_id(aup_id)
        if not aup:
            return None
        new_aup = Aup(
            aup.name,
            markdown,
            html,
            aup.conditions,
            aup.entitlement,
            version=aup.version + 1,
        )
        inserted_id = self.insert_aup(new_aup)
        if not inserted_id:
            return None
        self.db["aup"].update_one(
            {"_id": ObjectId(aup_id)}, {"$set": {"actual_aup_id": inserted_id}}
        )
        self.db["aup"].update_many(
            {"actual_aup_id": ObjectId(aup_id)},
            {"$set": {"actual_aup_id": inserted_id}},
        )
        return inserted_id

    def set_aup_conditions(self, aup_id: Union[str, int], conditions: List[List[str]]):
        self.db["aup"].update_one(
            {"_id": ObjectId(aup_id)}, {"$set": {"conditions": conditions}}
        )
        self.db["aup"].update_many(
            {"actual_aup_id": ObjectId(aup_id)}, {"$set": {"conditions": conditions}}
        )

    def save_request(self, request: Request) -> str:
        delattr(request, "_id")
        request.status = request.status.value
        result = self.db["request"].insert_one(request.__dict__)
        return result.inserted_id

    def get_request_by_id(self, request_id: Union[str, int]) -> Optional[Request]:
        return self.__get_request({"_id": ObjectId(request_id)})

    def get_request_by_nonce(self, nonce: str) -> Optional[Request]:
        return self.__get_request({"nonce": nonce})

    def __get_request(self, m_filter: json):
        request_m = self.db["request"].find_one(m_filter)
        if not request_m:
            return None
        return Request(
            request_m["nonce"],
            request_m["user_id"],
            request_m["_id"],
            Status(request_m["status"]),
        )

    def make_request_success(self, request_id: Union[str, int]):
        self.db["request"].update_one(
            {"_id": ObjectId(request_id)}, {"$set": {"status": Status.SUCCESS.value}}
        )

    def make_request_invalid(self, request_id: Union[str, int]):
        self.db["request"].update_one(
            {"_id": ObjectId(request_id)}, {"$set": {"status": Status.INVALID.value}}
        )

    @staticmethod
    def __aup_list_mongo_to_python(aup_list):
        return [MongodbDatabase.__aup_mongo_to_python(aup) for aup in aup_list]

    @staticmethod
    def __aup_mongo_to_python(aup_m):
        aup = Aup(**aup_m)
        aup.acceptances = [Acceptance(**ac) for ac in aup.acceptances]
        return aup



import json
from copy import deepcopy
from enum import Enum


class Status(Enum):
    FAILURE = 0
    SUCCESS = 1
    INVALID = 2


class Aup:
    def __init__(
        self,
        name,
        markdown,
        html,
        conditions,
        entitlement,
        _id=None,
        actual_aup_id=None,
        additional_data=None,
        version=1,
        acceptances=None,
    ):
        if acceptances is None:
            acceptances = []
        self._id = str(_id)
        self.actual_aup_id = str(actual_aup_id) if actual_aup_id else None
        self.name = name
        self.markdown = markdown
        self.html = html
        self.version = version
        self.conditions = conditions
        self.entitlement = entitlement
        self.additional_data = additional_data
        self.acceptances = acceptances

    def add_acceptance(self, acceptance):
        self.acceptances.append(acceptance)

    def to_dict(self):
        temp = deepcopy(self)
        temp.acceptances = [ac.__dict__ for ac in temp.acceptances]
        return temp.__dict__

    def to_response_dict(self):
        temp = deepcopy(self)
        acceptances = []
        for ac in temp.acceptances:
            ac._id = ac.get_id()
            ac.aup_id = str(ac.aup_id)
            acceptances.append(ac)

        temp._id = temp.get_id()
        temp.actual_aup_id = temp.get_actual_aup_id()
        response_dict = temp.to_dict()
        response_dict.pop("markdown")
        return response_dict

    def get_id(self):
        return self._id

    def get_actual_aup_id(self):
        return self.actual_aup_id

    def __str__(self):
        return str(self.to_dict())


class Acceptance:
    def __init__(self, aup_id, user_id, date_time, _id=None):
        self._id = str(_id)
        self.aup_id = aup_id
        self.user_id = user_id
        self.date_time = date_time

    def get_id(self):
        return self._id


class Request:
    def __init__(self, nonce, user_id, _id=None, status=Status.FAILURE):
        self._id = str(_id)
        self.nonce = nonce
        self.user_id = user_id
        self.status = status

    def get_id(self) -> str:
        return self._id

    def __str__(self):
        return str(self.__dict__)


class Admin:
    def __init__(self, _id, ext_login):
        self._id = str(_id)
        self.ext_login = ext_login

    def get_id(self):
        return self._id

    def __str__(self):
        return str(self.__dict__)

    def to_json(self):
        return json.dumps({"_id": self._id, "ext_login": self.ext_login})


class Entity:
    def __init__(self, _id, name, entity_type):
        self.name = name

        """type:id"""
        self.type_id = f"{entity_type}:{str(_id)}"

    def get_id(self) -> str:
        return self.type_id.split(":", 1)[1]

    def get_entity_type(self) -> str:
        return self.type_id.split(":", 1)[0]

    def __str__(self):
        return str(self.__dict__)



import abc
from typing import Union, List, Optional

from aup_manager.models import Aup, Acceptance, Request


class DatabaseInterface(metaclass=abc.ABCMeta):
    @classmethod
    def __subclasshook__(cls, subclass):
        return (
            hasattr(subclass, "insert_aup")
            and callable(subclass.insert_aup)
            and hasattr(subclass, "insert_acceptance")
            and callable(subclass.insert_acceptance)
            and hasattr(subclass, "insert_acceptances")
            and callable(subclass.insert_acceptances)
            and hasattr(subclass, "get_all_aups_with_acceptances")
            and callable(subclass.get_all_aups_with_acceptances)
            and hasattr(subclass, "get_aup_with_acceptances_by_id")
            and callable(subclass.get_aup_with_acceptances_by_id)
            and hasattr(subclass, "get_all_aups")
            and callable(subclass.get_all_aups)
            and hasattr(subclass, "get_aups_by_entitlement")
            and callable(subclass.get_aups_by_entitlement)
            and hasattr(subclass, "get_aup_by_id")
            and callable(subclass.get_aup_by_id)
            and hasattr(subclass, "get_all_user_accepted_aups")
            and callable(subclass.get_all_user_accepted_aups)
            and hasattr(subclass, "get_aups_by_condition")
            and callable(subclass.get_aups_by_condition)
            and hasattr(subclass, "get_aups_by_condition_with_acceptances")
            and callable(subclass.get_aups_by_condition_with_acceptances)
            and hasattr(subclass, "get_user_accepted_aups_by_condition")
            and callable(subclass.get_user_accepted_aups_by_condition)
            and hasattr(subclass, "get_user_not_accepted_aups_by_condition")
            and callable(subclass.get_user_not_accepted_aups_by_condition)
            and hasattr(subclass, "delete_aup_by_id")
            and callable(subclass.delete_aup_by_id)
            and hasattr(subclass, "set_aup_name")
            and callable(subclass.set_aup_name)
            and hasattr(subclass, "update_aup_text")
            and callable(subclass.update_aup_text)
            and hasattr(subclass, "set_aup_conditions")
            and callable(subclass.set_aup_conditions)
            and hasattr(subclass, "save_request")
            and callable(subclass.save_request)
            and hasattr(subclass, "get_request_by_id")
            and callable(subclass.get_request_by_id)
            and hasattr(subclass, "get_request_by_nonce")
            and callable(subclass.get_request_by_nonce)
            and hasattr(subclass, "make_request_success")
            and callable(subclass.make_request_success)
            and hasattr(subclass, "make_request_invalid")
            and callable(subclass.make_request_invalid)
            or NotImplemented
        )

    @abc.abstractmethod
    def insert_aup(self, aup: Aup) -> Union[str, int]:
        """Insert given Aup to database"""
        raise NotImplementedError

    @abc.abstractmethod
    def insert_acceptance(self, acceptance: Acceptance) -> Union[str, int]:
        """Insert given Acceptance to database"""
        raise NotImplementedError

    @abc.abstractmethod
    def insert_acceptances(
        self, acceptances: List[Acceptance]
    ) -> List[Union[str, int]]:
        """Insert multiple Acceptances to database"""
        raise NotImplementedError

    @abc.abstractmethod
    def get_all_aups_with_acceptances(self) -> List[Aup]:
        """Get all Aups with filled acceptance array"""
        raise NotImplementedError

    @abc.abstractmethod
    def get_aup_with_acceptances_by_id(self, _id: Union[str, int]) -> Optional[Aup]:
        """Get Aup by id with filled acceptance array"""
        raise NotImplementedError

    @abc.abstractmethod
    def get_all_aups(self) -> List[Aup]:
        """Get all Aups without acceptances"""
        raise NotImplementedError

    @abc.abstractmethod
    def get_aups_by_entitlement(self, entitlement: str) -> List[Aup]:
        """Get all Aups by entitlement"""
        raise NotImplementedError

    @abc.abstractmethod
    def get_aup_by_id(self, _id: Union[str, int]) -> Optional[Aup]:
        """Get Aup by id without acceptances"""
        raise NotImplementedError

    @abc.abstractmethod
    def get_all_user_accepted_aups(self, user_id: Union[int, str]) -> List[Aup]:
        """
        Get all Aups accepted by given user,
        acceptances array contains only user acceptances
        """
        raise NotImplementedError

    @abc.abstractmethod
    def get_aups_by_condition(self, condition: List[str]) -> List[Aup]:
        """Get Aups which fulfill condition without acceptances"""
        raise NotImplementedError

    @abc.abstractmethod
    def get_aups_by_condition_with_acceptances(self, condition: List[str]) -> List[Aup]:
        """Get Aups which fulfill condition with filled acceptance array"""
        raise NotImplementedError

    @abc.abstractmethod
    def get_user_accepted_aups_by_condition(
        self, user_id: Union[int, str], condition: List[str]
    ) -> List[Aup]:
        """Get Aups which fulfill condition and are accepted by given user,
        acceptances array contains only user acceptances"""
        raise NotImplementedError

    @abc.abstractmethod
    def get_user_not_accepted_aups_by_condition(
        self, user_id: Union[int, str], condition: List[str]
    ) -> List[Aup]:
        """
        Get Aups which fulfill condition and are not accepted by given user,
        acceptance array is empty
        """
        raise NotImplementedError

    @abc.abstractmethod
    def delete_aup_by_id(self, aup_id: Union[int, str]):
        """Delete Aup by id from database, deletes all acceptances assigned to Aup"""
        raise NotImplementedError

    @abc.abstractmethod
    def set_aup_name(self, aup_id: Union[str, int], new_name: str):
        """Set name to Aup and its previous versions"""
        raise NotImplementedError

    @abc.abstractmethod
    def update_aup_text(
        self, aup_id: Union[str, int], markdown: str, html: str
    ) -> Optional[Union[str, int]]:
        """
        Creates new version of Aup with new markdown and html texts,
        version is raised by 1
        """
        raise NotImplementedError

    @abc.abstractmethod
    def set_aup_conditions(self, aup_id: Union[str, int], conditions: List[str]):
        """Set conditions to Aup and its previous versions"""
        raise NotImplementedError

    @abc.abstractmethod
    def save_request(self, request: Request) -> str:
        """Save request to database"""
        raise NotImplementedError

    @abc.abstractmethod
    def get_request_by_id(self, request_id: Union[str, int]) -> Optional[Request]:
        """Get request by id"""
        raise NotImplementedError

    @abc.abstractmethod
    def get_request_by_nonce(self, nonce: str) -> Optional[Request]:
        """Get request by nonce"""
        raise NotImplementedError

    @abc.abstractmethod
    def make_request_success(self, request_id: Union[str, int]):
        """Update request result to success"""
        raise NotImplementedError

    @abc.abstractmethod
    def make_request_invalid(self, request_id: Union[str, int]):
        """Update request result to invalid"""
        raise NotImplementedError



from subprocess import check_call
import setuptools
from setuptools.command.install import install


class PostInstallCommand(install):
    """Post-installation for installation mode."""

    def run(self):
        check_call(
            "cd aup_manager/static && npm install && npm run compile_sass", shell=True
        )
        install.run(self)


setuptools.setup(
    name="aup_manager",
    python_requires=">=3.9",
    url="https://gitlab.ics.muni.cz/perun/perun-proxyidp/aup-manager.git",
    description="app for management of acceptable use policies with API for approving them",
    include_package_data=True,
    package_data={"": ["openapi-specification.yaml"]},
    packages=setuptools.find_packages(),
    install_requires=[
        "setuptools",
        "pymongo==4.6.1",  # for compatibility with proxyidp-gui
        "jsonpatch==1.33",
        "connexion[swagger-ui]==2.14.2",
        "markdown2==2.4.13",
        "Flask-pyoidc==3.14.3",
        "jwcrypto==1.5.6",
    ],
    extras_require={
        "perun": ["perun.connector==3.8.1"],
    },
    cmdclass={
        "install": PostInstallCommand,
    },
)



from aup_manager.connectors.ConnectorInterface import ConnectorInterface
from aup_manager.connectors.MongodbConnector import MongodbConnector
from aup_manager.connectors.PerunConnector import PerunConnector

__all__ = ["ConnectorInterface", "MongodbConnector", "PerunConnector"]
